'use strict';

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _sinon = require('sinon');

var _sinon2 = _interopRequireDefault(_sinon);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

_chai2.default.use(_sinonChai2.default);

describe('Class.onInherited', function () {
  describe("using class declarations", function () {
    it("is called when a class is inherited from", function () {
      var callback = _sinon2.default.spy();

      var Parent = function () {
        function Parent() {
          (0, _classCallCheck3.default)(this, Parent);
        }

        (0, _createClass3.default)(Parent, null, [{
          key: 'onInherited',
          value: function onInherited(child) {
            callback(child);
          }
        }]);
        return Parent;
      }();

      var Child = function (_Parent) {
        var _Child = function (_Parent2) {
          (0, _inherits3.default)(_Child, _Parent2);

          function _Child() {
            (0, _classCallCheck3.default)(this, _Child);
            return (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(_Child).apply(this, arguments));
          }

          return _Child;
        }(_Parent);

        var _Child2 = _Child;

        if (_Parent.hasOwnProperty("onInherited")) {
          if (typeof _Parent.onInherited == 'function') {
            _Child2 = _Parent.onInherited(_Child) || _Child;
          } else {
            throw new TypeError("Attempted to call onInherited, but it was not a function");
          }
        }

        return _Child2;
      }(Parent);

      (0, _chai.expect)(callback).to.have.been.calledWith(Child);
    });

    it("can redefine the value of the declared class", function () {
      var replacement = _sinon2.default.stub();

      var Parent = function () {
        function Parent() {
          (0, _classCallCheck3.default)(this, Parent);
        }

        (0, _createClass3.default)(Parent, null, [{
          key: 'onInherited',
          value: function onInherited(child) {
            return replacement;
          }
        }]);
        return Parent;
      }();

      var Child = function (_Parent3) {
        var _Child3 = function (_Parent4) {
          (0, _inherits3.default)(_Child3, _Parent4);

          function _Child3() {
            (0, _classCallCheck3.default)(this, _Child3);
            return (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(_Child3).apply(this, arguments));
          }

          return _Child3;
        }(_Parent3);

        var _Child4 = _Child3;

        if (_Parent3.hasOwnProperty("onInherited")) {
          if (typeof _Parent3.onInherited == 'function') {
            _Child4 = _Parent3.onInherited(_Child3) || _Child3;
          } else {
            throw new TypeError("Attempted to call onInherited, but it was not a function");
          }
        }

        return _Child4;
      }(Parent);

      (0, _chai.expect)(Child).to.equal(replacement);
    });
  });

  describe("using class expressions", function () {
    it("is called when a class is inherited from", function () {
      var callback = _sinon2.default.spy();

      var Parent = function () {
        function Parent() {
          (0, _classCallCheck3.default)(this, Parent);
        }

        (0, _createClass3.default)(Parent, null, [{
          key: 'onInherited',
          value: function onInherited(child) {
            callback(child);
          }
        }]);
        return Parent;
      }();

      var Child = function (_Parent5) {
        var _Child5 = function (_Parent6) {
          (0, _inherits3.default)(_Child5, _Parent6);

          function _Child5() {
            (0, _classCallCheck3.default)(this, _Child5);
            return (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(_Child5).apply(this, arguments));
          }

          return _Child5;
        }(_Parent5);

        var _Child6 = _Child5;

        if (_Parent5.hasOwnProperty("onInherited")) {
          if (typeof _Parent5.onInherited == 'function') {
            _Child6 = _Parent5.onInherited(_Child5) || _Child5;
          } else {
            throw new TypeError("Attempted to call onInherited, but it was not a function");
          }
        }

        return _Child6;
      }(Parent);

      (0, _chai.expect)(callback).to.have.been.calledWith(Child);
    });

    it("can redefine the value of the declared class", function () {
      var replacement = _sinon2.default.stub();

      var Parent = function () {
        function Parent() {
          (0, _classCallCheck3.default)(this, Parent);
        }

        (0, _createClass3.default)(Parent, null, [{
          key: 'onInherited',
          value: function onInherited(child) {
            return replacement;
          }
        }]);
        return Parent;
      }();

      var Child = function (_Parent7) {
        var _Child7 = function (_Parent8) {
          (0, _inherits3.default)(_Child7, _Parent8);

          function _Child7() {
            (0, _classCallCheck3.default)(this, _Child7);
            return (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(_Child7).apply(this, arguments));
          }

          return _Child7;
        }(_Parent7);

        var _Child8 = _Child7;

        if (_Parent7.hasOwnProperty("onInherited")) {
          if (typeof _Parent7.onInherited == 'function') {
            _Child8 = _Parent7.onInherited(_Child7) || _Child7;
          } else {
            throw new TypeError("Attempted to call onInherited, but it was not a function");
          }
        }

        return _Child8;
      }(Parent);

      (0, _chai.expect)(Child).to.equal(replacement);
    });
  });

  describe("classes without an onInherited callback", function () {
    it("still function properly", function () {
      var Parent = function () {
        function Parent() {
          (0, _classCallCheck3.default)(this, Parent);
        }

        (0, _createClass3.default)(Parent, null, [{
          key: 'number',
          value: function number() {
            return 7;
          }
        }]);
        return Parent;
      }();

      var Child = function (_Parent9) {
        var _Child9 = function (_Parent10) {
          (0, _inherits3.default)(_Child9, _Parent10);

          function _Child9() {
            (0, _classCallCheck3.default)(this, _Child9);
            return (0, _possibleConstructorReturn3.default)(this, (0, _getPrototypeOf2.default)(_Child9).apply(this, arguments));
          }

          (0, _createClass3.default)(_Child9, null, [{
            key: 'letter',
            value: function letter() {
              return "A";
            }
          }]);
          return _Child9;
        }(_Parent9);

        var _Child10 = _Child9;

        if (_Parent9.hasOwnProperty("onInherited")) {
          if (typeof _Parent9.onInherited == 'function') {
            _Child10 = _Parent9.onInherited(_Child9) || _Child9;
          } else {
            throw new TypeError("Attempted to call onInherited, but it was not a function");
          }
        }

        return _Child10;
      }(Parent);

      (0, _chai.expect)(Child.number()).to.equal(7);
      (0, _chai.expect)(Child.letter()).to.equal("A");
    });
  });
});